# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-05-06 08:15
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('appmail', '0002_add_template_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailtemplate',
            name='test_context',
            field=django.contrib.postgres.fields.jsonb.JSONField(
                default=dict,
                blank=True,
                help_text='Dummy JSON used for test rendering (set automatically on first save).')
            ,
        ),
    ]
